/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.client.client.ui.sinfo;

import java.util.ArrayList;
import java.util.List;

import org.dive4elements.river.client.client.ui.AbstractUIProvider;
import org.dive4elements.river.client.client.ui.DatacageWidget;
import org.dive4elements.river.client.client.ui.DatacageWidgetData;
import org.dive4elements.river.client.client.ui.PanelHelper;
import org.dive4elements.river.client.shared.model.Data;
import org.dive4elements.river.client.shared.model.DataItem;
import org.dive4elements.river.client.shared.model.DataList;
import org.dive4elements.river.client.shared.model.DefaultData;
import org.dive4elements.river.client.shared.model.DefaultDataItem;
import org.dive4elements.river.client.shared.model.InfrastructureServerClientXChange;
import org.dive4elements.river.client.shared.model.InfrastructureServerClientXChange.Element;
import org.dive4elements.river.client.shared.model.User;

import com.google.gwt.core.client.GWT;
import com.smartgwt.client.types.ListGridFieldType;
import com.smartgwt.client.util.SC;
import com.smartgwt.client.widgets.Button;
import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.Label;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.grid.ListGrid;
import com.smartgwt.client.widgets.grid.ListGridField;
import com.smartgwt.client.widgets.grid.ListGridRecord;
import com.smartgwt.client.widgets.grid.events.RecordClickEvent;
import com.smartgwt.client.widgets.grid.events.RecordClickHandler;
import com.smartgwt.client.widgets.layout.HLayout;
import com.smartgwt.client.widgets.layout.Layout;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.widgets.tree.TreeNode;

/**
 * @author Domenico Nardi Tironi
 *
 */
public class FloodInfrastructurePanel extends AbstractUIProvider {
    private static final long serialVersionUID = 1L;

    private static final String datakey = "flood_infrastructure";
    private ListGrid elements = new ListGrid();

    private final User m_user;

    private final InfrastructureServerClientXChange xchangeHelper = new InfrastructureServerClientXChange();

    private Button submit;

    public FloodInfrastructurePanel(final User user) {
        this.m_user = user;
    }

    protected final ListGrid createTable(final Layout root, final DataList data, final String width) {

        final Label title = new Label(data.get(0).getDescription());
        title.setHeight("35px");

        this.elements.setWidth(width);
        this.elements.setHeight(300);
        this.elements.setShowHeaderContextMenu(false);
        this.elements.setCanReorderFields(false);
        this.elements.setCanSort(true);
        this.elements.setSortField("to");
        this.elements.setCanResizeFields(false);

        this.elements.setCanEdit(false);

        final ListGridField groupIdField = new ListGridField("group_id", "groupId");
        groupIdField.setHidden(true);

        final ListGridField groupField = createField("group_label", "Infrastruktur BWaStr");// this.MSG.uinfo_vegetation_zones_label());

        final ListGridField typeIdField = new ListGridField("type_id", "typeId");
        typeIdField.setHidden(true);

        final ListGridField typeField = createField("type_label", "Typ/Bezeichnung");// this.MSG.uinfo_vegetation_zones_label());

        final ListGridField removeField = PanelHelper.createRemoveField(this.elements, GWT.getHostPageBaseURL() + this.MSG.removeFeature());
        this.elements.addRecordClickHandler(new RecordClickHandler() { // adding another custom record-Remove-Handler which is not included in the
            @Override
            public void onRecordClick(final RecordClickEvent event) {
                if (event.getField().getName().equals(removeField.getName())) {
                    final ListGridRecord r = event.getRecord();
                    final Element infrastr = new Element(r.getAttribute("group_id"), r.getAttribute("group_label"), r.getAttribute("type_id"),
                            r.getAttribute("type_label"));
                    FloodInfrastructurePanel.this.xchangeHelper.removeObject(infrastr);

                    validateInput();
                }
            }
        });
        this.elements.setFields(groupIdField, groupField, typeIdField, typeField, removeField);

        root.setWidth(width);
        root.addMember(title);
        root.addMember(this.elements);
        root.addMember(PanelHelper.getSpacer(3));
        root.addMember(PanelHelper.getSpacer(3));
        return this.elements;
    }

    private ListGridField createField(final String id, final String label) {
        final ListGridField field = new ListGridField(id, label);// this.MSG.uinfo_vegetation_zones_label());
        field.setType(ListGridFieldType.TEXT);
        field.setWidth("*"); // 245
        field.setCanSort(false);
        field.setCanDragResize(true);
        field.setCanEdit(false);
        return field;
    }

    @Override
    public final VLayout create(final DataList data) {
        final VLayout layout = new VLayout();

        final List<Data> items = data.getAll();
        final Data str = getData(items, datakey);
        final DataItem[] strItems = str.getItems();
        if (strItems.length > 0)
            this.xchangeHelper.parseAndAdd(strItems[0].getStringValue());

        final Canvas helper = createHelper(data);
        if (helper != null)
            this.helperContainer.addMember(helper);

        this.submit = (Button) getNextButton();

        final VLayout root = new VLayout();
        root.setWidth(450);
        createWidget(root, data);

        layout.addMember(root);
        layout.addMember(this.submit);

        return layout;
    }

    private void validateInput() {
        if (this.xchangeHelper.getItems().size() > 0)
            this.submit.enable();
        else
            this.submit.disable();
    }

    private Canvas createHelper(final DataList dataList) {

        final DatacageWidgetData data = new DatacageWidgetData(this.artifact, this.m_user, "sinfo_floodduration_infrastructures", "load-system:true", false);

        final DatacageWidget datacage = new DatacageWidget(data);

        final Button plusBtn = new Button("Hinzufügen"); // this.MSG.uinfo_inundation_duration_set_vegetation_zone()
        plusBtn.addClickHandler(new ClickHandler() {
            @Override
            public void onClick(final ClickEvent event) {
                handlePlusClicked(datacage);
            }
        });

        final VLayout helperLayout = new VLayout();
        helperLayout.addMember(datacage);
        helperLayout.addMember(plusBtn);

        return helperLayout;
    }

    private void handlePlusClicked(final DatacageWidget datacage) {
        final List<TreeNode> selection = datacage.getPlainSelection();
        if (selection == null || selection.isEmpty()) {
            SC.say(FloodInfrastructurePanel.this.MSG.warning());
            return;
        }

        for (final TreeNode selectedNode : selection) {
            final String groupId = selectedNode.getAttribute("group_id"); // node-names from meta-data.xml
            final String groupLabel = selectedNode.getAttribute("group_label");
            final String typeId = selectedNode.getAttribute("type_id");
            final String typeLabel = selectedNode.getAttribute("type_label");

            if (isEmpty(groupId) || isEmpty(groupLabel) || isEmpty(typeId) || isEmpty(typeLabel))
                continue;

            final InfrastructureServerClientXChange.Element infrastr = new InfrastructureServerClientXChange.Element(groupId, groupLabel, typeId, typeLabel);
            if (this.xchangeHelper.containsObject(infrastr))
                continue;

            this.xchangeHelper.addObject(infrastr);

            this.elements.addData(createEntry(infrastr));
        }
        validateInput();
    }

    private boolean isEmpty(final String test) {
        if (test == null)
            return true;
        if (test.trim().equals(""))
            return true;
        return false;
    }

    public final ListGridRecord createEntry(final InfrastructureServerClientXChange.Element row) {
        final ListGridRecord r = new ListGridRecord();
        r.setAttribute("group_id", row.getGroupId());
        r.setAttribute("group_label", row.getGroupLabel());
        r.setAttribute("type_id", row.getTypeId());
        r.setAttribute("type_label", row.getTypeLabel());
        return r;
    }

    public void createWidget(final Layout root, final DataList data) {

        this.elements = createTable(root, data, "420");

        for (final Element item : this.xchangeHelper.getItems())
            this.elements.addData(createEntry(item));
        validateInput();
    }

    @Override
    public Canvas createOld(final DataList dataList) {
        final HLayout layout = new HLayout();
        layout.setWidth("400px");
        final VLayout vLayout = new VLayout();
        vLayout.setWidth(130);
        final Label label = new Label(dataList.getLabel());
        label.setWidth("200px");
        label.setHeight("25px");

        final List<Data> items = dataList.getAll();
        final Data str = getData(items, datakey);
        final DataItem[] strItems = str.getItems();

        final List<Element> items1 = InfrastructureServerClientXChange.parse(strItems[0].getStringValue());

        final StringBuilder builder = new StringBuilder();

        for (final Element infrastr : items1) {
            builder.append(infrastr.getGroupLabel()).append(" - ").append(infrastr.getTypeLabel()).append("; ");
        }
        if (builder.length() > 1)
            builder.deleteCharAt(builder.length() - 2);
        else
            builder.append(" - keine Auswahl - (will never appear because the workflow is being disabled when the table is empty) ");
        final Label dateLabel = new Label(builder.toString());
        dateLabel.setHeight("20px");
        vLayout.addMember(dateLabel);

        final Canvas back = getBackButton(dataList.getState());
        layout.addMember(label);
        layout.addMember(vLayout);
        layout.addMember(back);

        return layout;
    }

    @Override
    protected final Data[] getData() {
        final List<Data> data = new ArrayList<Data>();

        final ListGridRecord[] lgr = this.elements.getRecords();
        if (lgr.length == 0) {
        }
        final String dataStr = this.xchangeHelper.parseListToDataString();

        final DataItem item = new DefaultDataItem(datakey, null,
                dataStr.equals("") ? "Non-Empty Value -> createOld would be called (not necessary, because workflow is disabled when table is empty)"
                        : dataStr);
        data.add(new DefaultData(datakey, null, null, new DataItem[] { item }));
        return data.toArray(new Data[data.size()]);
    }

}